@@ -32,11 +32,8 @@ class QuIPModifier(Modifier):
 
     Lifecycle:
         - on_initialize
-            - infer SpinQuantMappings & NormMappings
-            - as needed, create transform schemes for R1, R2, R3, & R4
+            - as needed, create transform schemes for V (input) and U (output)
         - on_start
-            - normalize embeddings
-            - fuse norm layers into subsequent Linear layers
             - apply TransformConfig
                 - fuse transforms into weights for mergeable transforms
                 - add hooks for online transforms
@@ -44,6 +41,9 @@ class QuIPModifier(Modifier):
         - on_end
         - on_finalize
 
+    :param rotations: which rotation schemes to apply to the model. Including `"v"` will
+        rotate the input side of weights, and including `"u"` will rotate the output
+        side of weights (note that v does not require u and vice-versa)
     :param transform_type: The type of transform to apply to the model.
         `"hadamard"` has the least performance cost but only supports sizes which are
         powers of power of two.
@@ -54,17 +54,24 @@ class QuIPModifier(Modifier):
     :param learnable: If true, attach gradients to transform weights for training
     :param precision: Precision at which all transforms should be applied. This applies
         to both weight fusing and online rotations
+    :param transform_block_size: Block size to use for rotation matrices. The model's
+        hidden_size must be evenly divisible by transform_block_size.
+        Layers will be transformed by a block-diagonal matrix where each block is a
+        matrix of this size.
+        If None is provided, model's hidden_size will be used
     :param ignore: Modules to ignore when attaching transforms
     :param transform_config: Optional transform config for overriding provided arguments
     """  # noqa: E501
 
+    rotations: List[Literal["v", "u"]] = Field(default_factory=lambda: ["v", "u"])
     transform_type: Literal["hadamard", "random-hadamard", "random-matrix"] = Field(
         default="random-hadamard"
     )
-    targets: Union[List[str], str] = Field(default="str")
+    targets: Union[List[str], str] = Field(default="Linear")
     randomize: bool = Field(default=False)
     learnable: bool = Field(default=False)
     precision: TorchDtype = Field(default=torch.float64)
+    transform_block_size: Optional[int] = Field(default=None)
     ignore: Union[str, List[str]] = Field(default="lm_head")
 
     # optional override for more fine-grained control
@@ -77,6 +84,12 @@ class QuIPModifier(Modifier):
             raise NotImplementedError(f"{info.field_name} is not supported right now")
         return value
 
+    @field_validator("rotations", mode="before")
+    def validate_lowercase_list(cls, value):
+        if isinstance(value, list):
+            value = [v.lower() if isinstance(v, str) else v for v in value]
+        return value
+
     def on_initialize(self, state: State, **kwargs) -> bool:
         if self.transform_config is not None:
             return True
@@ -111,45 +124,54 @@ class QuIPModifier(Modifier):
         return True
 
     def _create_config(self) -> TransformConfig:
-        return TransformConfig(
-            config_groups={
-                "v": TransformScheme(
-                    type=self.transform_type,
-                    apply=[
-                        TransformArgs(
-                            targets=self.targets,
-                            location="input",  # non-mergable
-                            ignore=self.ignore,
-                        ),
-                        TransformArgs(
-                            targets=self.targets,
-                            location="weight_input",
-                            inverse=True,
-                            ignore=self.ignore,
-                        ),
-                    ],
-                    randomize=self.randomize,
-                    requires_grad=self.learnable,
-                    precision=self.precision,
+        config_groups = dict()
+        if "v" in self.rotations:
+            config_groups["v"] = self._create_v_scheme()
+        if "u" in self.rotations:
+            config_groups["u"] = self._create_u_scheme()
+
+        return TransformConfig(config_groups=config_groups)
+
+    def _create_v_scheme(self) -> TransformScheme:
+        return TransformScheme(
+            type=self.transform_type,
+            head_dim=self.transform_block_size,
+            apply=[
+                TransformArgs(
+                    targets=self.targets,
+                    location="input",  # non-mergable
+                    ignore=self.ignore,
                 ),
-                "u": TransformScheme(
-                    type=self.transform_type,
-                    apply=[
-                        TransformArgs(
-                            targets=self.targets,
-                            location="weight_output",
-                            ignore=self.ignore,
-                        ),
-                        TransformArgs(
-                            targets=self.targets,
-                            location="output",  # non-mergable
-                            inverse=True,
-                            ignore=self.ignore,
-                        ),
-                    ],
-                    randomize=self.randomize,
-                    requires_grad=self.learnable,
-                    precision=self.precision,
+                TransformArgs(
+                    targets=self.targets,
+                    location="weight_input",
+                    inverse=True,
+                    ignore=self.ignore,
                 ),
-            }
+            ],
+            randomize=self.randomize,
+            requires_grad=self.learnable,
+            precision=self.precision,
+        )
+
+    def _create_u_scheme(self) -> TransformScheme:
+        return TransformScheme(
+            type=self.transform_type,
+            head_dim=self.transform_block_size,
+            apply=[
+                TransformArgs(
+                    targets=self.targets,
+                    location="weight_output",
+                    ignore=self.ignore,
+                ),
+                TransformArgs(
+                    targets=self.targets,
+                    location="output",  # non-mergable
+                    inverse=True,
+                    ignore=self.ignore,
+                ),
+            ],
+            randomize=self.randomize,
+            requires_grad=self.learnable,
+            precision=self.precision,
         )
