@@ -1,17 +1,20 @@
-from .base import RecipeBase
+"""
+Recipe system for defining and managing compression workflows.
+
+Provides the recipe framework for specifying compression
+configurations, including metadata tracking, recipe parsing, and
+workflow orchestration. Supports stage-based execution and flexible
+parameter management for complex compression pipelines.
+"""
+
 from .metadata import DatasetMetaData, LayerMetaData, ModelMetaData, ParamMetaData
-from .modifier import RecipeModifier
 from .recipe import Recipe, RecipeArgsInput, RecipeInput, RecipeStageInput
-from .stage import RecipeStage
 
 __all__ = [
     "DatasetMetaData",
     "ParamMetaData",
     "LayerMetaData",
     "ModelMetaData",
-    "RecipeBase",
-    "RecipeModifier",
-    "RecipeStage",
     "Recipe",
     "RecipeInput",
     "RecipeStageInput",
